
var path = require("path");
var fs = require("fs");
var cheerio = require('cheerio');

module.exports = function(config){

    return {
        compile:function(tpl_dir,data){
            var app_module = data.app_module;
            var deployer = build_deployer(config,app_module);

            //var app_id = req.params["app_id"];
            var page_content = fs.readFileSync(tpl_dir);
            const $ = cheerio.load(page_content,{
                decodeEntities: false
            });

            $("[x-bind]").each(function(){
                var binds = $(this).attr("x-bind");

                if(binds){
                    binds = binds.split(".");
                }

                var d=app_module;
                for(var i=0;i<binds.length;i++){
                    d = d[binds[i]];
                }
                if(d){
                    if($(this).is("input")){
                        $(this).val(d);
                    }else{
                        $(this).html(d);
                    }
                    
                }                
            });

            $("head").append("<script>var global_app_name='"+app_module.app_name+"';var global_app_id='"+(data.app_id || "")+"';</script>");

            $("script[x-holder='style']").after(deployer.write_styles()).remove();
            $("script[x-holder='script']").after(deployer.write_scripts()).remove();


            return $.html();
        }
    }
}

function build_deployer(config,app_module){
    var app_config = app_module.config;

    var templates = {};
    
    var pre_url = app_module.pre_url 
    //? "/"+app_module.pre_url : "";

    function write_styles(){
        var style_src = [];
        (app_config.styles||[]).forEach(function(style) {
            if(config.style_alias[style]){
                style = config.style_alias[style];
            }

            //不是绝对路径则自动补全
            if(style.indexOf("/") !== 0 && style.indexOf("http") !== 0 ){

                style = pre_url+"/"+app_module.app_name+"/" + style;
            }

            style_src.push("<link rel='stylesheet' href='"+style+"'></link>");
            
        });

        return style_src.join("\n");
    }

    function get_cpmt_path(app_module,cpmt_name){
        return pre_url+"/"+app_module.app_name+"/"+(app_config.component_dir||config.component_dir)+"/"+cpmt_name+".js";
    }

    function get_view_path(app_module,cpmt_name){
        return "/"+app_module.app_name+"/"+(app_config.view_dir||config.view_dir)+"/"+cpmt_name+".html";
    }

    function write_scripts(){
        var script_src = [];

        function push_libs(libs){
            if(!libs) return ;
            libs.forEach(function(script) {
                if(config.script_alias[script]){
                    script = config.script_alias[script];
                    //数组
                    if(script.push){
                        push_libs(script);
                        return ;
                    }
                }

                //不是绝对路径则自动补全
                if(script.indexOf("/") !== 0 && script.indexOf("http") !== 0){
                    script = pre_url+"/"+app_module.app_name+"/" + script;
                }

                script_src.push("<script type='text/javascript' src='"+script+"'></script>");
            });
        }

        push_libs(app_config.libs);


        (app_config.components||[]).forEach(function(cpmt) {
            var cpmt_path;
            if(config.script_alias[cpmt]){
                cpmt_path = config.script_alias[cpmt];
            }else{
                cpmt_path = get_cpmt_path(app_module,cpmt);                
            }

            script_src.push("<script type='text/javascript' src='"+cpmt_path+"'></script>");

            var view_path = get_view_path(app_module,cpmt);


            var view_content = fs.readFileSync(path.join(config.root_dir,app_module.app_root || app_module.platform_root ,view_path),'utf-8');
            script_src.push(view_content);

        });


        return script_src.join("\n");
    }


    return  {
        write_styles:write_styles,
        write_scripts:write_scripts
    }
}
